@@ -306,25 +306,41 @@ ldsm_mount_should_ignore (GUnixMountEntry *mount)
          * expose this in a way that allows it to be used for this
          * purpose
          */
-                 
+
+        /* We also ignore network filesystems */
+
         const gchar *ignore_fs[] = {
+                "adfs",
+                "afs",
                 "auto",
                 "autofs",
+                "autofs4",
+                "cifs",
+                "cxfs",
                 "devfs",
                 "devpts",
                 "ecryptfs",
+                "gfs",
+                "gfs2",
                 "kernfs",
                 "linprocfs",
+                "linsysfs",
+                "lustre",
+                "lustre_lite",
+                "ncpfs",
+                "nfs",
+                "nfs4",
+                "nfsd",
+                "ocfs2",
                 "proc",
                 "procfs",
                 "ptyfs",
+                "rpc_pipefs",
                 "selinuxfs",
-                "linsysfs",
+                "smbfs",
                 "sysfs",
                 "tmpfs",
                 "usbfs",
-                "nfsd",
-                "rpc_pipefs",
                 "zfs",
                 NULL
         };
@@ -546,11 +562,7 @@ ldsm_mounts_changed (GObject  *monitor,
         mounts = g_unix_mounts_get (time_read);
         g_hash_table_foreach_remove (ldsm_notified_hash,
                                      ldsm_is_hash_item_not_in_mounts, mounts);
-#if GLIB_CHECK_VERSION (2, 28, 0)
         g_list_free_full (mounts, (GDestroyNotify) g_unix_mount_free);
-#else
-        g_list_foreach (mounts, (GFunc) g_unix_mount_free, NULL);
-#endif
 
         /* check the status now, for the new mounts */
         ldsm_check_all_mounts (NULL);
