@@ -90,21 +90,6 @@ static const gchar* indicator_off_icon_names[] = {
 	"kbd-capslock-off"
 };
 
-//#define noMSDKX
-
-#ifdef MSDKX
-static FILE *logfile;
-
-static void msd_keyboard_log_appender(const char file[], const char function[], int level, const char format[], va_list args)
-{
-	time_t now = time (NULL);
-	fprintf (logfile, "[%08ld,%03d,%s:%s/] \t", now,
-		 level, file, function);
-	vfprintf (logfile, format, args);
-	fflush (logfile);
-}
-#endif
-
 static void
 g_strv_behead (gchar **arr)
 {
@@ -598,11 +583,6 @@ msd_keyboard_xkb_init (MsdKeyboardManager * kbd_manager)
 
 	msd_keyboard_update_indicator_icons ();
 
-#ifdef MSDKX
-	xkl_set_debug_level (200);
-	logfile = fopen ("/tmp/msdkx.log", "a");
-	xkl_set_log_appender (msd_keyboard_log_appender);
-#endif
 	manager = kbd_manager;
 	mate_settings_profile_start ("xkl_engine_get_instance");
 	xkl_engine = xkl_engine_get_instance (display);
