@@ -132,7 +132,7 @@ enum {
         NUMBER_OF_SIGNALS
 };
 
-static guint msd_smartcard_manager_signals[NUMBER_OF_SIGNALS];
+static guint msd_smartcard_manager_signals[NUMBER_OF_SIGNALS] = { 0 };
 
 G_DEFINE_TYPE (MsdSmartcardManager,
                msd_smartcard_manager,
@@ -340,11 +340,6 @@ msd_smartcard_manager_init (MsdSmartcardManager *manager)
                                        g_str_equal,
                                        (GDestroyNotify) g_free,
                                        (GDestroyNotify) g_object_unref);
-
-        if (!g_thread_supported ()) {
-                g_thread_init (NULL);
-        }
-
 }
 
 static void
@@ -1095,7 +1090,7 @@ msd_smartcard_manager_worker_watch_for_and_process_event (MsdSmartcardManagerWor
                                                           GError                    **error)
 {
         PK11SlotInfo *slot;
-        CK_SLOT_ID slot_id, *key;
+        CK_SLOT_ID slot_id, *key = NULL;
         int slot_series, card_slot_series;
         MsdSmartcard *card;
         GError *processing_error;
@@ -1261,9 +1256,9 @@ msd_smartcard_manager_create_worker (MsdSmartcardManager  *manager,
         worker = msd_smartcard_manager_worker_new (write_fd);
         worker->module = manager->priv->module;
 
-        *worker_thread = g_thread_create ((GThreadFunc)
+        *worker_thread = g_thread_new ("MsdSmartcardManagerWorker", (GThreadFunc)
                                           msd_smartcard_manager_worker_run,
-                                          worker, FALSE, NULL);
+                                          worker);
 
         if (*worker_thread == NULL) {
                 msd_smartcard_manager_worker_free (worker);
@@ -1335,8 +1330,6 @@ main (int   argc,
         g_log_set_always_fatal (G_LOG_LEVEL_ERROR
                                 | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);
 
-        g_type_init ();
-
         g_message ("creating instance of 'smartcard manager' object...");
         manager = msd_smartcard_manager_new (NULL);
         g_message ("'smartcard manager' object created successfully");
